package org.ow2.orchestra.definition.activity;

import java.util.Date;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.jbpm.pvm.internal.model.ObservableElementImpl;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.facade.data.def.ProcessActivityDefinitionData;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.handlers.FinishedInstanceHandler;
import org.ow2.orchestra.util.EnvTool;

public class Process extends Scope {

  private static final Logger LOG = Logger.getLogger(Process.class.getName());

  public Process() {
    super();
  }

  @Override
  public void signal(final BpelExecution execution, final String signal, final Map<String, Object> parameters) {
    if (signal == null) {
      super.signal(execution, signal, parameters);
      return;
    }
    // execute source to log end of scope ?
    executeSources(execution);
    if (execution.isActive()) {
      // Call the FinishedInstanceHandler
      FinishedInstanceHandler handler = EnvTool.get(FinishedInstanceHandler.class, FinishedInstanceHandler.DEFAULT_KEY);
      handler.handleFinishedInstance(execution.getProcessInstanceData());

      final ObservableElementImpl parent = execution.getNode().getParent();
      if (parent == null) {
        if (LOG.isLoggable(Level.FINE)) {
          LOG.fine("top level scope ends the bpel process execution");
        }
        if (execution.getParent() != null) {
          throw new IllegalStateException("Child execution reaches the end of process !");
        }
        if (execution.getWaitingExecutions() != null && !execution.getWaitingExecutions().isEmpty()) {
          throw new IllegalStateException(
              "Execution reaches the end of process, but child executions are still waiting for a trigger !");
        }
        execution.end();
        execution.terminate();
      }
    }
  }

  @Override
  public ProcessActivityDefinitionData createDefinitionData(final BpelProcess bpelProcess) {
    return new ProcessActivityDefinitionData(bpelProcess.getProcessDefinitionData(), uuid, null, name, null);
  }

  @Override
  public void beforeRunning(final BpelExecution bpelExecution) {

    long activityCount = bpelExecution.getNextActivityCount();
    ActivityInstanceUUID activityInstanceUUID = bpelExecution.getProcessInstanceUUID();
    bpelExecution.setStartedDate(new Date());
    bpelExecution.setActivityInstanceUUID(activityInstanceUUID);
    bpelExecution.setActivityPosition(activityCount);

    EnvTool.getRecorder().recordInstanceStarted(bpelExecution.getProcessInstanceData());

  }

  @Override
  public void afterRunned(final BpelExecution bpelExecution) {
    EnvTool.getRecorder().recordInstanceEnded(bpelExecution.getProcessInstanceData());
  }
}
