package org.ow2.orchestra.definition.element;

import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.datatype.Duration;

import org.jbpm.pvm.internal.model.NodeImpl;
import org.ow2.novabpm.util.Misc;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.runtime.BpelExecution;


public class OnAlarm {

  private static Logger log = Logger.getLogger(OnAlarm.class.getName());

  private String uuid = Misc.getUniqueId("onAlarm");

  private Expression durationExpression;

  private Expression deadlineExpression;

  protected NodeImpl activity;

  public void setDurationExpression(final Expression durationExpression) {
    this.durationExpression = durationExpression;
  }
  public void setDeadlineExpression(final Expression deadlineExpression) {
    this.deadlineExpression = deadlineExpression;
  }
  public NodeImpl getActivity() {
    return activity;
  }
  public void setActivity(final NodeImpl activity) {
    this.activity = activity;
  }
  public Expression getDurationExpression() {
    return durationExpression;
  }
  public Expression getDeadlineExpression() {
    return deadlineExpression;
  }



  public static Date computeTimer(final BpelExecution execution, final Expression deadlineExpression,
      final Expression durationExpression) {
    Date date = null;

    if (deadlineExpression != null) {
      date = deadlineExpression.getEvaluator().evaluateDeadline(execution).toGregorianCalendar().getTime();
      if (log.isLoggable(Level.FINE)) {
        log.fine("waiting until " + date);
      }

    } else if (durationExpression != null) {
      Duration duration = durationExpression.getEvaluator().evaluateDuration(execution);
      date = new Date(execution.getStartedDate().getTime());
      duration.addTo(date);
      if (log.isLoggable(Level.FINE)) {
        log.fine("waiting for " + duration);
      }

    } else {
      throw new OrchestraRuntimeException("invalid timer definition");
    }

    return date;
  }

  public String getUuid() {
    return uuid;
  }
}
