/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.facade.def.full.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.def.full.ActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.ActivityWithChildrenFullDefinition;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

/**
 *
 * @author Gael Lalire
 */
public abstract class ActivityWithChildrenFullDefinitionImpl extends ActivityFullDefinitionImpl implements
  ActivityWithChildrenFullDefinition {

  private static final long serialVersionUID = 8901738568448948365L;

  /**
   * Warning this field is not persisted so you have to use getter instead of
   * direct reference.
   */
  protected List<ActivityDefinitionUUID> enclosedActivitiesUUID;

  protected List<ActivityFullDefinition> enclosedActivities;

  protected ActivityWithChildrenFullDefinitionImpl() {
  }

  public ActivityWithChildrenFullDefinitionImpl(final ActivityDefinitionUUID uuid, final ProcessDefinitionUUID processUUID,
    final String name) {
    super(uuid, processUUID, name);
    this.enclosedActivities = new ArrayList<ActivityFullDefinition>();
  }

  public ActivityWithChildrenFullDefinitionImpl(final ActivityWithChildrenFullDefinition activityFullDefinition) {
    super(activityFullDefinition);
    enclosedActivities = CopyUtil.fullCopyList(activityFullDefinition.getEnclosedActivities());
  }

  public List<ActivityFullDefinition> getEnclosedActivities() {
    return this.enclosedActivities;
  }

  public void addEnclosedActivity(final ActivityFullDefinition enclosedActivity) {
    this.enclosedActivities.add(enclosedActivity);
    getEnclosedActivitiesUUID().add(enclosedActivity.getUUID());
  }

  public List<ActivityDefinitionUUID> getEnclosedActivitiesUUID() {
    if (enclosedActivitiesUUID == null) {
      List<ActivityDefinitionUUID> enclosedActivitiesUUID = new ArrayList<ActivityDefinitionUUID>();
      for (ActivityFullDefinition enclosedActivity : enclosedActivities) {
        enclosedActivitiesUUID.add(enclosedActivity.getUUID());
      }
      this.enclosedActivitiesUUID = enclosedActivitiesUUID;
    }
    return enclosedActivitiesUUID;
  }

}
