/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.def.CorrelationSetDefinition;
import org.ow2.orchestra.facade.def.PartnerLinkDefinition;
import org.ow2.orchestra.facade.def.VariableDefinition;
import org.ow2.orchestra.facade.def.full.ScopeActivityFullDefinition;
import org.ow2.orchestra.facade.def.impl.ScopeActivityDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

/**
 *
 * @author Alfonso Fuca
 */
public class ScopeActivityFullDefinitionImpl extends ActivityWithSingleChildFullDefinitionImpl implements
  ScopeActivityFullDefinition {

  private static final long serialVersionUID = -6085964762975213580L;

  private boolean exitOnStandardFault;

  protected List<CorrelationSetDefinition> correlationSets = null;

  protected List<VariableDefinition> variables = null;

  protected List<PartnerLinkDefinition> partnerLinks = null;

  private List<String> messageExchanges = null;

  protected ScopeActivityFullDefinitionImpl() {
  }

  public ScopeActivityFullDefinitionImpl(final ProcessDefinitionUUID processUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name) {
    super(activityDefinitionUUID, processUUID, name);
    this.variables = new ArrayList<VariableDefinition>();
    this.correlationSets = new ArrayList<CorrelationSetDefinition>();
    this.partnerLinks = new ArrayList<PartnerLinkDefinition>();
    this.messageExchanges = new ArrayList<String>();
  }

  public ScopeActivityFullDefinitionImpl(final ScopeActivityFullDefinition scopeActivityFullDefinition) {
    super(scopeActivityFullDefinition);
    this.variables = new ArrayList<VariableDefinition>();
    this.variables = CopyUtil.copyList(scopeActivityFullDefinition.getVariables());
    this.correlationSets = new ArrayList<CorrelationSetDefinition>();
    this.correlationSets = CopyUtil.copyList(scopeActivityFullDefinition.getCorrelationSets());
    this.partnerLinks = new ArrayList<PartnerLinkDefinition>();
    this.partnerLinks = CopyUtil.copyList(scopeActivityFullDefinition.getPartnerLinks());
    this.messageExchanges = new ArrayList<String>();
    this.messageExchanges.addAll(scopeActivityFullDefinition.getMessageExchanges());
  }

  public ScopeActivityDefinitionImpl copy() {
    return new ScopeActivityDefinitionImpl(this);
  }

  public ScopeActivityFullDefinitionImpl fullCopy() {
    return new ScopeActivityFullDefinitionImpl(this);
  }

  @Override
  public ActivityType getType() {
    return ActivityType.SCOPE;
  }

  public boolean getExitOnStandardFault() {
    return exitOnStandardFault;
  }

  public void setExitOnStandardFault(final boolean exitOnStandardFault) {
    this.exitOnStandardFault = exitOnStandardFault;
  }

  public List<VariableDefinition> getVariables() {
    return variables;
  }

  public void setVariableDefinition(final List<VariableDefinition> variableDefinitions) {
    this.variables = CopyUtil.copyList(variableDefinitions);
  }

  public List<CorrelationSetDefinition> getCorrelationSets() {
    return correlationSets;
  }

  public void setCorrelationSetDefinition(final List<CorrelationSetDefinition> correlationSetDefinitions) {
    this.correlationSets = CopyUtil.copyList(correlationSetDefinitions);
  }

  public List<PartnerLinkDefinition> getPartnerLinks() {
    return partnerLinks;
  }

  public void setPartnerLinkDefinition(final List<PartnerLinkDefinition> partnerLinkDefinitions) {
    this.partnerLinks = CopyUtil.copyList(partnerLinkDefinitions);
  }

  public List<String> getMessageExchanges() {
    return messageExchanges;
  }

  public void setMessageExchange(final Set<String> messageExchangeList) {
    for (String messageExchange : messageExchangeList) {
      this.messageExchanges.add(messageExchange);
    }
  }
}
