/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.impl;

import java.util.List;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.def.ActivityDefinition;
import org.ow2.orchestra.facade.def.Source;
import org.ow2.orchestra.facade.def.Target;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;

/**
 *
 * @author Gael Lalire
 */
public abstract class ActivityDefinitionImpl extends DefinitionRecordImpl implements ActivityDefinition {

  private static final long serialVersionUID = -6522865558817129517L;

  protected ActivityDefinitionUUID uuid;

  protected boolean suppressJoinFailure;

  protected List<Source> sourceList;

  protected List<Target> targetList;

  protected String targetsJoinConditionExpressionLanguage;

  protected String targetsJoinConditionBooleanExpression;

  public ActivityDefinitionImpl(final ActivityDefinition src) {
    super(src);
    uuid = src.getUUID();
    suppressJoinFailure = src.isSuppressJoinFailure();
    sourceList = CopyUtil.copyList(src.getSourceList());
    targetList = CopyUtil.copyList(src.getTargetList());
    targetsJoinConditionExpressionLanguage = src.getTargetsJoinConditionExpressionLanguage();
    targetsJoinConditionBooleanExpression = src.getTargetsJoinConditionBooleanExpression();
  }

  public ActivityDefinitionUUID getUUID() {
    return uuid;
  }

  public abstract ActivityType getType();

  /**
   * @return the suppressJoinFailure
   */
  public boolean isSuppressJoinFailure() {
    return suppressJoinFailure;
  }

  public List<Source> getSourceList() {
    return sourceList;
  }

  public List<Target> getTargetList() {
    return targetList;
  }

  public String getTargetsJoinConditionBooleanExpression() {
    return targetsJoinConditionBooleanExpression;
  }

  public String getTargetsJoinConditionExpressionLanguage() {
    return targetsJoinConditionExpressionLanguage;
  }

}
