/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.full.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.CorrelationSetInitialization;
import org.ow2.orchestra.facade.runtime.PartnerLinkUpdate;
import org.ow2.orchestra.facade.runtime.VariableUpdate;
import org.ow2.orchestra.facade.runtime.full.ScopeActivityFullInstance;
import org.ow2.orchestra.facade.runtime.impl.ScopeActivityInstanceImpl;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;

/**
 *
 * @author Alfonso Fuca
 */
public class ScopeActivityFullInstanceImpl extends ActivityWithSingleChildFullInstanceImpl implements ScopeActivityFullInstance {

  private static final long serialVersionUID = 62610068746337790L;

  /** all runtime variables of this scope. */
  protected List<VariableUpdate> variables = null;

  protected List<CorrelationSetInitialization> correlationSet = null;

  protected List<PartnerLinkUpdate> partnerLink = null;

  protected ScopeActivityFullInstanceImpl() {
  }

  public ScopeActivityFullInstanceImpl(final ActivityInstanceUUID uuid, final ActivityDefinitionUUID definitionUUID,
    final ProcessDefinitionUUID processUUID,
    final ProcessInstanceUUID instanceUUID) {
    super(uuid, definitionUUID, processUUID, instanceUUID);
    this.variables = new ArrayList<VariableUpdate>();
    this.correlationSet = new ArrayList<CorrelationSetInitialization>();
    this.partnerLink = new ArrayList<PartnerLinkUpdate>();
  }

  public ScopeActivityFullInstanceImpl(final ScopeActivityFullInstance scopeActivityFullInstance) {
    super(scopeActivityFullInstance);
    variables = new ArrayList<VariableUpdate>();
    for (VariableUpdate variable : scopeActivityFullInstance.getVariables()) {
      variables.add(new VariableUpdateFullImpl(variable));
    }
    this.correlationSet = new ArrayList<CorrelationSetInitialization>();
    for (CorrelationSetInitialization correlationSetInitialization : scopeActivityFullInstance.getCorrelationSet()) {
      correlationSet.add(new CorrelationSetInitializationFullImpl(correlationSetInitialization));
    }
    this.partnerLink = new ArrayList<PartnerLinkUpdate>();
    for (PartnerLinkUpdate partnerLinkUpdate : scopeActivityFullInstance.getPartnerLink()) {
      partnerLink.add(new PartnerLinkUpdateFullImpl(partnerLinkUpdate));
    }
  }

  public List<VariableUpdate> getVariables() {
    return variables;
  }

  @Override
  public ActivityType getType() {
    return ActivityType.SCOPE;
  }

  public ScopeActivityFullInstanceImpl fullCopy() {
    return new ScopeActivityFullInstanceImpl(this);
  }

  public ScopeActivityInstanceImpl copy() {
    return new ScopeActivityInstanceImpl(this);
  }

  public List<CorrelationSetInitialization> getCorrelationSet() {
    return correlationSet;
  }

  public List<PartnerLinkUpdate> getPartnerLink() {
    return partnerLink;
  }

}
