/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.facade.runtime.impl;

import org.ow2.orchestra.facade.runtime.ActivityInstance;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;

/**
 *
 * @author Gael Lalire
 */
public abstract class ActivityInstanceImpl extends RuntimeRecordImpl implements ActivityInstance {

  private static final long serialVersionUID = 9188864525348045460L;

  protected ActivityInstanceUUID uuid;
  protected ActivityDefinitionUUID definitionUUID;

  public ActivityInstanceImpl(final ActivityInstance record) {
    super(record);
    this.uuid = record.getUUID();
    this.definitionUUID = record.getDefinitionUUID();
  }

  @Override
  public String toString() {
    String st = this.getClass().getName() + "[uuid: " + getUUID() + ", processDefinitionUUID: " + getProcessDefinitionUUID()
      + ", processUUID: " + getProcessInstanceUUID() + ", startedDate: " + getStartedDate() + ", endedDate: " + getEndedDate();
    st += "]\n";
    return st;
  }

  public ActivityInstanceUUID getUUID() {
    return uuid;
  }

  public ActivityDefinitionUUID getDefinitionUUID() {
    return definitionUUID;
  }

  @Override
  public final int hashCode() {
    return uuid.hashCode() + super.hashCode();
  }

  @Override
  public final boolean equals(final Object obj) {
    if (obj == null) {
      return false;
    }
    if (!(obj.getClass().equals(this.getClass()))) {
      return false;
    }
    if (!super.equals(obj)) {
      return false;
    }
    if (!uuid.equals(((ActivityInstanceImpl) obj).getUUID())) {
      return false;
    }
    return true;
  }

  public abstract ActivityType getType();

}
