/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.runtime.CorrelationSetInitialization;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.w3c.dom.Node;

/**
 *
 * @author Gael Lalire
 */
public class CorrelationSetInitializationImpl implements CorrelationSetInitialization {

  private static final long serialVersionUID = -2069580133901045271L;

  private ActivityInstanceUUID activityUUID;

  private Date date;

  private String name;

  protected Map<QName, Node> propertyValues;

  public CorrelationSetInitializationImpl(final CorrelationSetInitialization record) {
    activityUUID = record.getActivityUUID();
    date = new Date(record.getDate().getTime());
    name = record.getName();
    this.propertyValues = new HashMap<QName, Node>();
    for (Entry<QName, Node> entry : record.getPropertyValues().entrySet()) {
      propertyValues.put(entry.getKey(), entry.getValue().cloneNode(true));
    }
  }

  public ActivityInstanceUUID getActivityUUID() {
    return activityUUID;
  }

  public Date getDate() {
    return date;
  }

  public String getName() {
    return name;
  }

  public Map<QName, Node> getPropertyValues() {
    return propertyValues;
  }

  public CorrelationSetInitialization copy() {
    return new CorrelationSetInitializationImpl(this);
  }

}
