/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.impl;

import java.util.List;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.CorrelationSetInitialization;
import org.ow2.orchestra.facade.runtime.PartnerLinkUpdate;
import org.ow2.orchestra.facade.runtime.ScopeActivityInstance;
import org.ow2.orchestra.facade.runtime.VariableUpdate;

/**
 *
 * @author Alfonso Fuca
 */
public class ScopeActivityInstanceImpl extends ActivityWithSingleChildInstanceImpl implements ScopeActivityInstance {

  private static final long serialVersionUID = 8770698137420371016L;

  /** all runtime variables of this scope. */
  protected List<VariableUpdate> variables = null;

  protected List<CorrelationSetInitialization> correlationSet = null;

  protected List<PartnerLinkUpdate> partnerLink = null;

  public ScopeActivityInstanceImpl(final ScopeActivityInstance record) {
    super(record);
    variables = CopyUtil.copyList(record.getVariables());
    this.correlationSet = CopyUtil.copyList(record.getCorrelationSet());
    this.partnerLink = CopyUtil.copyList(record.getPartnerLink());
  }

  @Override
  public ActivityType getType() {
    return ActivityType.SCOPE;
  }

  @Override
  public String toString() {
    String st = this.getClass().getName() + "[uuid: " + getUUID() + ", processDefinitionUUID: " + getProcessDefinitionUUID()
      + ", processUUID: " + getProcessInstanceUUID() + ", startedDate: " + getStartedDate() + ", endedDate: " + getEndedDate()
      + ", activityType: " + getType();
    st += "]\n";
    return st;
  }

  public List<VariableUpdate> getVariables() {
    return variables;
  }

  public List<CorrelationSetInitialization> getCorrelationSet() {
    return correlationSet;
  }

  public List<PartnerLinkUpdate> getPartnerLink() {
    return partnerLink;
  }

}
