/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.jbpm.pvm.internal.util.XmlUtil;
import org.jbpm.pvm.internal.xml.Parse;
import org.jbpm.pvm.internal.xml.Parser;
import org.ow2.orchestra.definition.activity.Throw;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.w3c.dom.Element;

public class ThrowBinding extends ActivityBinding {

  /** log. */
  private static Logger log = Logger.getLogger(ThrowBinding.class.getName());


  public ThrowBinding() {
    super("throw", ActivityType.THROW);
  }

  /**
   * Parses the given dom element to build a Node and a behaviour.
   *
   * @param element
   *            dom element to parse
   * @param parse
   *            parse
   * @param parser
   *            parser
   * @return the build behaviour!
   */
  public Object parse(final Element element, final Parse parse, final Parser parser) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("parsing element = " + element);
    }
    Throw throw_ = new Throw();
    setEnclosingScope(throw_, parse);
    parseStandardAttributes(element, throw_, parse);
    parseStandardElements(element, throw_, parse);

    // attributes
    throw_.setFaultName(XmlUtil.attributeQName(element, "faultName"));
    throw_.setFaultVariable(XmlUtil.attribute(element, "faultVariable"));


    org.jbpm.pvm.model.Node activityNode = createActivityNode(element, parse, parser,
        throw_);
    return activityNode;
  }

}
