/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.jbpm.pvm.internal.model.NodeImpl;
import org.jbpm.pvm.internal.util.XmlUtil;
import org.jbpm.pvm.internal.xml.Parse;
import org.jbpm.pvm.internal.xml.Parser;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.activity.Wait;
import org.ow2.orchestra.facade.def.ForDefinition;
import org.ow2.orchestra.facade.def.UntilDefinition;
import org.ow2.orchestra.facade.def.full.ActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.WaitActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.impl.WaitActivityFullDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.ForDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.UntilDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.lang.Expression;
import org.w3c.dom.Element;

/**
 * @author Pascal Verdage
 */
public class WaitBinding extends ActivityBinding {

  private static Logger log = Logger.getLogger(WaitBinding.class.getName());

  public WaitBinding() {
    super("wait", ActivityType.WAIT);
  }

  public Object parse(final Element element, final Parse parse, final Parser parser) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("parsing element = " + element);
    }
    Wait waitObject = new Wait();
    NodeImpl waitNode = createActivityNode(element, parse, parser, waitObject);
//    waitNode.setLocalScope(true);

    setEnclosingScope(waitObject, parse);
    parseStandardAttributes(element, waitObject, parse);

    DefinitionKeeper definitionKeeper = parse.findObject(DefinitionKeeper.class);
    ActivityFullDefinition activityFullDefinition = definitionKeeper.getActivityFullDefinition();
    ForDefinition forDefinition = new ForDefinitionImpl();
    UntilDefinition untilDefinition = new UntilDefinitionImpl();

    parseStandardElements(element, waitObject, parse);

    BpelProcess bpelProcess = parse.findObject(BpelProcess.class);
    Element deadlineElement = XmlUtil.element(element, "until");
    Element durationElement = XmlUtil.element(element, "for");
    if (durationElement != null && deadlineElement != null) {
      parse.addProblem("Error in element: " + element + " only one element 'for' or 'until' should be used.");
    }
    if (durationElement != null) {
      Expression expression = PickBinding.parseExpression(durationElement, bpelProcess);
      waitObject.setDurationExpression(expression);

      // Definition
      forDefinition = new ForDefinitionImpl(expression.getLanguage(), expression.getText());
    } else if (deadlineElement != null) {
      Expression expression = PickBinding.parseExpression(deadlineElement, bpelProcess);
      waitObject.setDeadlineExpression(expression);

      // Definition
      untilDefinition = new UntilDefinitionImpl(expression.getLanguage(), expression.getText());
    } else {
      parse.addProblem("Error in element: " + element + " one element 'for' or 'until' should be used.");
    }

    ((WaitActivityFullDefinition) activityFullDefinition).addForDefinition(forDefinition);
    ((WaitActivityFullDefinition) activityFullDefinition).addUntilDefinition(untilDefinition);

    return waitNode;
  }
  @Override
  public ActivityFullDefinition createActivityFullDefinition(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name, final Parse parse) {
    return new WaitActivityFullDefinitionImpl(processDefinitionUUID, activityDefinitionUUID, name);
  }

}
