/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.persistence.db;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.namespace.QName;

import org.ow2.novabpm.util.Misc;
import org.ow2.orchestra.facade.def.ProcessState;
import org.ow2.orchestra.facade.def.full.ActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.ProcessFullDefinition;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.ProcessInstanceState;
import org.ow2.orchestra.facade.runtime.full.ActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.ProcessFullInstance;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.persistence.QuerierDbSession;
import org.ow2.orchestra.services.Querier;

/**
 *
 * @author Gael Lalire
 */
public class AbstractDbQuerier implements Querier {

  private final QuerierDbSession dbSession;

  private Map<ProcessInstanceUUID, ProcessFullInstance> processFullInstanceByProcessInstanceUUIDMap;

  private Map<ActivityInstanceUUID, ActivityFullInstance> activityFullInstanceByActivityInstanceUUIDMap;

  public AbstractDbQuerier(final QuerierDbSession dbSession) {
    super();
    this.dbSession = dbSession;
    processFullInstanceByProcessInstanceUUIDMap = new HashMap<ProcessInstanceUUID, ProcessFullInstance>();
    activityFullInstanceByActivityInstanceUUIDMap = new HashMap<ActivityInstanceUUID, ActivityFullInstance>();
  }

  protected QuerierDbSession getQuerierDbSession() {
    return this.dbSession;
  }

  public ActivityFullInstance getActivityInstance(final ActivityInstanceUUID activityInstanceUUID) {
    ActivityFullInstance activityFullInstance = activityFullInstanceByActivityInstanceUUIDMap
        .get(activityInstanceUUID);
    if (activityFullInstance == null) {
      activityFullInstance = getQuerierDbSession().getActivityInstance(
          activityInstanceUUID);
      if (activityFullInstance == null) {
        Misc.unreachableStatement("activity instance not found " + activityInstanceUUID);
      }
      activityFullInstanceByActivityInstanceUUIDMap.put(activityFullInstance.getUUID(), activityFullInstance);
    }
    return activityFullInstance;
  }

  public void addProcessInstance(final ProcessFullInstance processFullInstance) {
    processFullInstanceByProcessInstanceUUIDMap.put(processFullInstance.getUUID(), processFullInstance);
    activityFullInstanceByActivityInstanceUUIDMap.put(processFullInstance.getUUID(), processFullInstance);
  }

  public void addActivityInstance(final ActivityFullInstance activityFullInstance) {
    activityFullInstanceByActivityInstanceUUIDMap.put(activityFullInstance.getUUID(), activityFullInstance);
  }

  public ProcessFullInstance getProcessInstance(
      final ProcessInstanceUUID processInstanceUUID) {
    ProcessFullInstance processFullInstance = processFullInstanceByProcessInstanceUUIDMap
        .get(processInstanceUUID);
    if (processFullInstance == null) {
      processFullInstance = getQuerierDbSession().getProcessInstance(
          processInstanceUUID);
    }
    return processFullInstance;
  }

  public void removeProcessInstance(final ProcessFullInstance processFullInstance) {
    processFullInstanceByProcessInstanceUUIDMap.remove(processFullInstance.getProcessInstanceUUID());
  }

  public Set<ActivityFullInstance> findActivityInstances(
      final ProcessInstanceUUID processUUID) {
    return getQuerierDbSession().findActivityInstance(processUUID);
  }

  public Set<ActivityFullInstance> findActivityInstances(
      final ProcessInstanceUUID processDefinitionUUID, final ActivityState activityState) {
    return getQuerierDbSession().findActivityInstance(processDefinitionUUID, activityState);
  }

  public Set<ProcessFullInstance> findProcessInstances() {
    return getQuerierDbSession().findProcessInstance();
  }

  public Set<ProcessFullInstance> findProcessInstances(
      final ProcessDefinitionUUID processDefinitionUUID) {
    return getQuerierDbSession().findProcessInstance(processDefinitionUUID);
  }

  public Set<ProcessFullInstance> findProcessInstances(
      final ProcessDefinitionUUID processDefinitionUUID, final ProcessInstanceState processInstanceState) {
    return getQuerierDbSession().findProcessInstance(processDefinitionUUID, processInstanceState);
  }

  public Set<ProcessFullInstance> findProcessInstances(final ProcessInstanceState processInstanceState) {
    return getQuerierDbSession().findProcessInstance(processInstanceState);
  }

  public ActivityFullDefinition getActivityDefinition(final ActivityDefinitionUUID activityDefinitionUUID) {
    return getQuerierDbSession().getActivityDefinition(activityDefinitionUUID);
  }

  public List<ProcessFullDefinition> findProcessDefinitions(final QName name, final ProcessState state) {
    return getQuerierDbSession().findProcessDefinition(name, state);
  }
  public List<ProcessFullDefinition> findProcessDefinitions(final ProcessState state) {
    return getQuerierDbSession().findProcessDefinition(state);
  }
  public List<ProcessFullDefinition> findProcessDefinitions(final QName name) {
    return getQuerierDbSession().findProcessDefinition(name);
  }
  public List<ProcessFullDefinition> findProcessDefinitions() {
    return getQuerierDbSession().findProcessDefinition();
  }

  public ProcessFullDefinition getProcessDefinition(final ProcessDefinitionUUID processDefinitionUUID) {
    return getQuerierDbSession().getProcessDefinition(processDefinitionUUID);
  }

}


