/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.persistence.db;

import java.util.Date;

import org.ow2.novabpm.util.Misc;
import org.ow2.orchestra.facade.data.runtime.ActivityInstanceData;
import org.ow2.orchestra.facade.data.runtime.CorrelationSetInitializedData;
import org.ow2.orchestra.facade.data.runtime.ForeachInitializedData;
import org.ow2.orchestra.facade.data.runtime.PartnerLinkUpdateData;
import org.ow2.orchestra.facade.data.runtime.ProcessInstanceData;
import org.ow2.orchestra.facade.data.runtime.TimerData;
import org.ow2.orchestra.facade.data.runtime.TransitionConditionUpdateData;
import org.ow2.orchestra.facade.data.runtime.VariableUpdateData;
import org.ow2.orchestra.facade.def.full.ProcessFullDefinition;
import org.ow2.orchestra.facade.runtime.WaitingActivity;
import org.ow2.orchestra.facade.runtime.full.ActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.ActivityWithChildrenFullInstance;
import org.ow2.orchestra.facade.runtime.full.ActivityWithSingleChildFullInstance;
import org.ow2.orchestra.facade.runtime.full.FlowActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.ForeachActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.PickActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.ProcessFullInstance;
import org.ow2.orchestra.facade.runtime.full.ScopeActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.WaitActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.impl.AssignActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.CorrelationSetInitializationFullImpl;
import org.ow2.orchestra.facade.runtime.full.impl.EmptyActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.FlowActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ForeachActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.IfActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.InvokeActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.PartnerLinkUpdateFullImpl;
import org.ow2.orchestra.facade.runtime.full.impl.PickActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ProcessFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ReceiveActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.RepeatUntilActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ReplyActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ScopeActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.SequenceActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.TransitionConditionUpdateFullImpl;
import org.ow2.orchestra.facade.runtime.full.impl.UnknownActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.VariableUpdateFullImpl;
import org.ow2.orchestra.facade.runtime.full.impl.WaitActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.WaitingActivityFullImpl;
import org.ow2.orchestra.facade.runtime.full.impl.WhileActivityFullInstanceImpl;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.persistence.QuerierDbSession;
import org.ow2.orchestra.services.Archivable;
import org.ow2.orchestra.services.Recorder;

/**
 *
 * @author Gael Lalire
 */
public class DbJournal extends AbstractDbQuerier implements Recorder {

  public DbJournal(final QuerierDbSession dbSession) {
    super(dbSession);
  }

  public void recordInstanceStarted(final ProcessInstanceData processInstanceData) {
    final ProcessDefinitionUUID processUUID = processInstanceData.getProcessDefinitionData().getUUID();
    final ProcessInstanceUUID instanceUUID = processInstanceData.getUUID();
    Misc.checkArgsNotNull(processUUID, instanceUUID);

    final ProcessFullInstance processFullInstance = new ProcessFullInstanceImpl(instanceUUID, processUUID);
    addProcessInstance(processFullInstance);
    getQuerierDbSession().save(processFullInstance);
  }

  public void recordInstanceEnded(final ProcessInstanceData processInstanceData) {
    final ProcessInstanceUUID instanceUUID = processInstanceData.getUUID();
    Misc.checkArgsNotNull(instanceUUID);

    final ProcessFullInstance processFullInstance = getProcessInstance(instanceUUID);
    removeProcessInstance(processFullInstance);
    processFullInstance.end();
  }

  public void recordActivityStarted(final ActivityInstanceData activityInstanceData) {
    final ProcessDefinitionUUID processUUID = activityInstanceData.getActivityDefinitionData().getProcessDefinitionData()
      .getUUID();
    final ProcessInstanceUUID instanceUUID = activityInstanceData.getProcessInstanceData().getUUID();
    final ActivityInstanceUUID activityUUID = activityInstanceData.getUUID();
    final ActivityDefinitionUUID activityDefinitionUUID = activityInstanceData.getActivityDefinitionData().getUUID();
    final ActivityInstanceUUID encloserInstanceUUID = activityInstanceData.getEncloserUUID();
    Misc.checkArgsNotNull(processUUID, instanceUUID, activityUUID);

    ActivityFullInstance activityFullInstance;
    switch (activityInstanceData.getActivityDefinitionData().getType()) {
      case ASSIGN:
        activityFullInstance = new AssignActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case SCOPE:
        activityFullInstance = new ScopeActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case PROCESS:
        activityFullInstance = new ProcessFullInstanceImpl(instanceUUID, processUUID);
        break;
      case SEQUENCE:
        activityFullInstance = new SequenceActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case RECEIVE:
        activityFullInstance = new ReceiveActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case REPLY:
        activityFullInstance = new ReplyActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case INVOKE:
        activityFullInstance = new InvokeActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case WAIT:
        activityFullInstance = new WaitActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case PICK:
        activityFullInstance = new PickActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case EMPTY:
        activityFullInstance = new EmptyActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case IF:
        activityFullInstance = new IfActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case REPEAT_UNTIL:
        activityFullInstance = new RepeatUntilActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case FOR_EACH:
        activityFullInstance = new ForeachActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case WHILE:
        activityFullInstance = new WhileActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case FLOW:
        activityFullInstance = new FlowActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case COMPENSATE_SCOPE:
      case EXIT:
      case EXTENSION_ACTIVITY:
      case RETHROW:
      case THROW:
      case VALIDATE:
      default:
        activityFullInstance = new UnknownActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
    }

    addActivityInstance(activityFullInstance);

    if (encloserInstanceUUID == null) {
      // process direct child
      final ProcessFullInstance processFullInstance = getProcessInstance(instanceUUID);
      processFullInstance.setEnclosedActivity(activityFullInstance);
    } else if (encloserInstanceUUID.equals(activityFullInstance.getUUID())) {
      Misc.unreachableStatement("EncloserUUID (" + encloserInstanceUUID
        + ") cannot have the same UUID that the activityFullInstance (" + activityFullInstance.getUUID());
    } else {
      // parent activity
      final ActivityFullInstance encloserActivityFullInstance = getActivityInstance(encloserInstanceUUID);
      if (encloserActivityFullInstance instanceof ActivityWithChildrenFullInstance) {
        ((ActivityWithChildrenFullInstance) encloserActivityFullInstance).addEnclosedActivity(activityFullInstance);
      } else if (encloserActivityFullInstance instanceof ActivityWithSingleChildFullInstance) {
        ((ActivityWithSingleChildFullInstance) encloserActivityFullInstance).setEnclosedActivity(activityFullInstance);
      } else {
        Misc.unreachableStatement("EncloserUUID (" + encloserInstanceUUID + ") " + "has no possible enclosed activity. "
          + "It cannot be the parent of activityFullInstance (" + activityFullInstance.getUUID());
      }
    }
  }

  public void recordActivityEnded(final ActivityInstanceData activityInstanceData) {
    final ActivityInstanceUUID activityUUID = activityInstanceData.getUUID();
    final ActivityFullInstance activityFullInstance = getActivityInstance(activityUUID);
    activityFullInstance.end();
  }

  public void remove(final Archivable archivable) {
    if (archivable instanceof ProcessFullInstance) {
      remove((ProcessFullInstance) archivable);
    } else if (archivable instanceof ProcessFullDefinition) {
      remove((ProcessFullDefinition) archivable);
    } else {
      Misc.unreachableStatement();
    }
  }

  public void remove(final ProcessFullInstance processFullInstance) {
    Misc.checkArgsNotNull(processFullInstance);
    if (getProcessInstance(processFullInstance.getUUID()) != null) {
      getQuerierDbSession().delete(processFullInstance);
    }
  }

  public void remove(final ProcessFullDefinition processFullDefinition) {
    Misc.checkArgsNotNull(processFullDefinition);
    if (getProcessDefinition(processFullDefinition.getUUID()) != null) {
      getQuerierDbSession().delete(processFullDefinition);
    }
  }

  public void recordTimerStarted(final TimerData timerData) {
    final ActivityFullInstance activityFullInstance = getActivityInstance(timerData.getActivityInstanceUUID());
    WaitingActivity waitingActivity = new WaitingActivityFullImpl(timerData.getActivityDefinitionUUID(), timerData
      .getExpectedStartedDate());
    switch (activityFullInstance.getType()) {
      case PICK:
        ((PickActivityFullInstance) activityFullInstance).addWaitingActivity(waitingActivity);
        break;
      case WAIT:
        WaitActivityFullInstance waitActivityFullInstance = (WaitActivityFullInstance) activityFullInstance;
        Misc.badStateIfNotNull(waitActivityFullInstance.getWaitingActivity(), "wait can only own one timer");
        waitActivityFullInstance.setWaitingActivity(waitingActivity);
        break;
      default:
        Misc.unreachableStatement("Only wait and pick activity can own timers");
    }
  }

  public void recordForeachInitialized(final ForeachInitializedData foreachInitializedData) {
    ForeachActivityFullInstance foreachActivityFullInstance = (ForeachActivityFullInstance) getActivityInstance(foreachInitializedData
      .getActivityInstanceUUID());
    foreachActivityFullInstance.update(foreachInitializedData.getStartValue(), foreachInitializedData.getStopValue(),
      foreachInitializedData.getCompletionConditionValue());
  }

  public void recordVariableUpdated(final VariableUpdateData variableUpdateData) {
    // following scopeActivityFullInstance may be a scope or a process
    ScopeActivityFullInstance scopeActivityFullInstance = (ScopeActivityFullInstance) getActivityInstance(variableUpdateData
      .getScopeActivityUUID());

    scopeActivityFullInstance.getVariables().add(
      new VariableUpdateFullImpl(variableUpdateData.getActivityUUID(), new Date(), variableUpdateData.getName(),
        variableUpdateData.getValue()));
  }

  public void recordCorrelationSetInitialized(final CorrelationSetInitializedData correlationSetInitializedData) {
    ScopeActivityFullInstance scopeActivityFullInstance = (ScopeActivityFullInstance) getActivityInstance(correlationSetInitializedData
      .getScopeActivityUUID());

    scopeActivityFullInstance.getCorrelationSet().add(
      new CorrelationSetInitializationFullImpl(correlationSetInitializedData.getActivityUUID(), new Date(),
        correlationSetInitializedData.getName(), correlationSetInitializedData.getPropertyValues()));
  }

  public void recordPartnerLinkUpdated(final PartnerLinkUpdateData partnerLinkUpdateData) {
    ScopeActivityFullInstance scopeActivityFullInstance = (ScopeActivityFullInstance) getActivityInstance(partnerLinkUpdateData
      .getScopeActivityUUID());

    if (scopeActivityFullInstance.getPartnerLink() == null || partnerLinkUpdateData == null) {
      Misc.unreachableStatement();
    }

    scopeActivityFullInstance.getPartnerLink().add(
      new PartnerLinkUpdateFullImpl(partnerLinkUpdateData.getActivityUUID(), new Date(), partnerLinkUpdateData.getName(),
        partnerLinkUpdateData.getPartnerLinkUpdateType(), partnerLinkUpdateData.getPartnerLinkReference()));
  }

  public void recordTransitionConditionUpdated(final TransitionConditionUpdateData transitionConditionUpdateData) {
    FlowActivityFullInstance flowActivityFullInstance = (FlowActivityFullInstance) getActivityInstance(transitionConditionUpdateData
      .getFlowActivityUUID());

    flowActivityFullInstance.getTransitionConditionUpdateList().add(
      new TransitionConditionUpdateFullImpl(transitionConditionUpdateData.getStatus(),
        transitionConditionUpdateData.getName(), new Date()));
  }

  public void recordProcessDeployed(final ProcessFullDefinition processFullDefinition) {
    getQuerierDbSession().save(processFullDefinition);
  }

  public void recordProcessUndeployed(final ProcessDefinitionUUID processDefinitionUUID) {
    ProcessFullDefinition processFullDefinition = getProcessDefinition(processDefinitionUUID);
    processFullDefinition.setUndeployedDate(new Date());
  }
}
