/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.persistence.db;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.hibernate.FlushMode;
import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.type.Type;
import org.jbpm.pvm.internal.hibernate.HibernatePvmDbSession;
import org.ow2.novabpm.util.hibernate.GenericEnumUserType;
import org.ow2.orchestra.facade.def.ProcessState;
import org.ow2.orchestra.facade.def.full.ActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.ProcessFullDefinition;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.ProcessInstanceState;
import org.ow2.orchestra.facade.runtime.full.ActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.ProcessFullInstance;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.persistence.QuerierDbSession;

/**
 *
 * @author Gael Lalire
 */
public class QuerierDbSessionImpl extends HibernatePvmDbSession implements QuerierDbSession {

  private static final Logger LOG = Logger.getLogger(QuerierDbSessionImpl.class.getName());

  public QuerierDbSessionImpl(final Session session) {
    setSession(session);
    session.setFlushMode(FlushMode.COMMIT);
  }

  public ActivityFullInstance getActivityInstance(final ActivityInstanceUUID activityInstanceUUID) {
    Query query = getSession().getNamedQuery("findActivityInstanceByUUID");
    query.setString("uuid", activityInstanceUUID.toString());
    return (ActivityFullInstance) query.uniqueResult();
  }

  public ProcessFullInstance getProcessInstance(final ProcessInstanceUUID processInstanceUUID) {
    Query query = getSession().getNamedQuery("findProcessInstanceByUUID");
    query.setString("uuid", processInstanceUUID.toString());
    return (ProcessFullInstance) query.uniqueResult();
  }

  public Set<ActivityFullInstance> findActivityInstance(
      final ProcessInstanceUUID processUUID) {
    Query query = getSession().getNamedQuery("findActivityInstancesWithProcessUUID");
    query.setString("processInstanceUUID", processUUID.toString());
    List<ActivityFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ActivityFullInstance>(results);
    } else {
      return new HashSet<ActivityFullInstance>();
    }
  }

  public Set<ActivityFullInstance> findActivityInstance(
      final ProcessInstanceUUID processInstanceUUID, final ActivityState activityState) {
    Query query;
    switch (activityState) {
      case RUNNING:
        query = getSession().getNamedQuery(
            "findActivityInstancesWithProcessUUIDAndActive");
        break;
      case FINISHED:
        query = getSession().getNamedQuery(
            "findActivityInstancesWithProcessUUIDAndInactive");
        break;
      default:
        query = getSession().getNamedQuery("findActivityInstancesWithProcessUUID");
    }
    query.setString("processInstanceUUID", processInstanceUUID.toString());
    List<ActivityFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ActivityFullInstance>(results);
    } else {
      return new HashSet<ActivityFullInstance>();
    }
  }

  public Set<ProcessFullInstance> findProcessInstance() {
    Query query = getSession().getNamedQuery("findProcessInstances");
    List<ProcessFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullInstance>(results);
    } else {
      return new HashSet<ProcessFullInstance>();
    }
  }

  public Set<ProcessFullInstance> findProcessInstance(
      final ProcessDefinitionUUID processDefinitionUUID) {
    Query query = getSession().getNamedQuery("findProcessInstancesWithProcessDefinitionUUID");
    query.setString("processDefinitionUUID", processDefinitionUUID.toString());
    List<ProcessFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullInstance>(results);
    } else {
      return new HashSet<ProcessFullInstance>();
    }
  }

  public Set<ProcessFullInstance> findProcessInstance(
      final ProcessDefinitionUUID processDefinitionUUID, final ProcessInstanceState processInstanceState) {
    Query query;
    switch (processInstanceState) {
      case RUNNING:
        query = getSession().getNamedQuery(
            "findProcessInstancesWithProcessDefinitionUUIDAndActive");
        break;
      case FINISHED:
        query = getSession().getNamedQuery(
            "findProcessInstancesWithProcessDefinitionUUIDAndInactive");
        break;
      default:
        query = getSession().getNamedQuery(
            "findProcessInstancesWithProcessDefinitionUUID");
    }
    query.setString("processDefinitionUUID", processDefinitionUUID.toString());
    List<ProcessFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullInstance>(results);
    } else {
      return new HashSet<ProcessFullInstance>();
    }
  }

  public Set<ProcessFullInstance> findProcessInstance(final ProcessInstanceState processInstanceState) {
    Query query;
    switch (processInstanceState) {
      case RUNNING:
        query = getSession().getNamedQuery("findProcessInstancesWithActive");
        break;
      case FINISHED:
        query = getSession().getNamedQuery("findProcessInstancesWithInactive");
        break;
      default:
        query = getSession().getNamedQuery("findProcessInstances");
    }
    List<ProcessFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullInstance>(results);
    } else {
      return new HashSet<ProcessFullInstance>();
    }
  }

  private static Type processStateUserType =
    Hibernate.custom(GenericEnumUserType.class, new String[]{"enumClass"}, new String[]{ProcessState.class.getName()});
  private static Type processInstanceStateUserType =
    Hibernate.custom(GenericEnumUserType.class, new String[]{"enumClass"}, new String[]{ProcessInstanceState.class.getName()});

  public List<ProcessFullDefinition> findProcessDefinition(final QName name, final ProcessState state) {
    Query query = getSession().getNamedQuery("findProcessDefinitionWithQNameAndState");
    query.setString("name", name.getLocalPart());
    query.setString("nameSpace", name.getNamespaceURI());
    query.setParameter("state", state, processStateUserType);

    List<ProcessFullDefinition> results = query.list();
    if (results != null) {
      return new ArrayList<ProcessFullDefinition>(results);
    } else {
      return new ArrayList<ProcessFullDefinition>();
    }
  }

  public List<ProcessFullDefinition> findProcessDefinition(final QName name) {
    Query query = getSession().getNamedQuery("findProcessDefinitionWithQName");
    query.setString("name", name.getLocalPart());
    query.setString("nameSpace", name.getNamespaceURI());

    List<ProcessFullDefinition> results = query.list();
    if (results != null) {
      return new ArrayList<ProcessFullDefinition>(results);
    } else {
      return new ArrayList<ProcessFullDefinition>();
    }
  }

  public List<ProcessFullDefinition> findProcessDefinition(final ProcessState state) {
    Query query = getSession().getNamedQuery("findProcessDefinitionWithState");
    query.setParameter("state", state, processStateUserType);

    List<ProcessFullDefinition> results = query.list();
    if (results != null) {
      return new ArrayList<ProcessFullDefinition>(results);
    } else {
      return new ArrayList<ProcessFullDefinition>();
    }
  }

  public List<ProcessFullDefinition> findProcessDefinition() {
    Query query = getSession().getNamedQuery("findProcessDefinition");

    List<ProcessFullDefinition> results = query.list();
    if (results != null) {
      return new ArrayList<ProcessFullDefinition>(results);
    } else {
      return new ArrayList<ProcessFullDefinition>();
    }
  }
  public ProcessFullDefinition getProcessDefinition(final ProcessDefinitionUUID processDefinitionUUID) {
    Query query = getSession().getNamedQuery("findProcessDefinitionByUUID");
    query.setString("uuid", processDefinitionUUID.toString());
    return (ProcessFullDefinition) query.uniqueResult();
  }

  public ActivityFullDefinition getActivityDefinition(final ActivityDefinitionUUID activityDefinitionUUID) {
    Query query = getSession().getNamedQuery("findActivityDefinitionByUUID");
    query.setString("uuid", activityDefinitionUUID.toString());
    return (ActivityFullDefinition) query.uniqueResult();
  }

}


