/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.runtime;

import org.jbpm.pvm.env.Environment;
import org.jbpm.pvm.internal.job.TimerImpl;
import org.jbpm.pvm.session.MessageSession;
import org.ow2.orchestra.services.AsyncFindExecToSignal;
import org.ow2.orchestra.services.MutexRepository;
import org.ow2.orchestra.services.Receiver;
import org.ow2.orchestra.services.MutexRepository.InstanceLock;

public class BpelTimerImpl extends TimerImpl {

  @Override
  public Boolean execute(final Environment environment) throws Exception {
    final BpelExecution bpelExecution = (BpelExecution) getExecution();
    final BpelExecution bpelInstance = bpelExecution.getProcessInstance();
    final InstanceLock lock = MutexRepository.lockInstance(bpelInstance);
    boolean res;
    try {
      res = super.execute(environment);
    } finally {
      MutexRepository.unlockInstance(lock);
    }
    if (!bpelInstance.isEnded()) {
      Receiver.addWaitingExecutions(bpelInstance);
      environment.get(MessageSession.class).send(
        new AsyncFindExecToSignal(bpelInstance, bpelInstance.getProcessDefinition().getQName())
      );
    }
    return res;
  }
}
