/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

/**
 *
 */
package org.ow2.orchestra.services;

import org.jbpm.pvm.env.Environment;
import org.jbpm.pvm.session.MessageSession;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.itf.Publisher;
import org.ow2.orchestra.var.Message;

class AsyncAssociateMessageCommand extends BpelMessageImpl<BpelExecution> {
  private final OperationKey operationKey;
  private final Message incomingMessage;
  private final MessageCarrier messageCarrier;

  public AsyncAssociateMessageCommand(final OperationKey operationKey,
      final Message incomingMessage, final MessageCarrier messageCarrier) {
    this.operationKey = operationKey;
    this.incomingMessage = incomingMessage;
    this.messageCarrier = messageCarrier;
  }

  public BpelExecution execute(final Environment environment) throws Exception {
    final Publisher servicePublisher = environment.get(Publisher.class);
    final Receiver receiver = servicePublisher.getReceiver(this.operationKey);
    if (receiver == null) {
      throw new OrchestraRuntimeException("impossible to find a receiver corresponding to: " + this.operationKey);
    }
    ExecElementToSignal execToSignal = receiver.handle(this.incomingMessage, environment, this.messageCarrier);

    if (execToSignal != null) {
      final BpelExecution currentInstance = execToSignal.getBpelExecution().getProcessInstance();
      environment.get(MessageSession.class).send(new AsyncSignalExec(currentInstance, this.operationKey.getProcessQName(), execToSignal));
      return execToSignal.getBpelExecution();
    }

    return null;
  }
}