/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.services;

import javax.xml.namespace.QName;

import org.jbpm.pvm.env.Environment;
import org.jbpm.pvm.internal.jobexecutor.JobDbSession;
import org.jbpm.pvm.session.MessageSession;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.MutexRepository.InstanceLock;
import org.ow2.orchestra.services.MutexRepository.ProcessLock;

public class AsyncFindExecToSignal extends BpelMessageImpl<Boolean> {


  protected AsyncFindExecToSignal() { }
  public AsyncFindExecToSignal(final BpelExecution instance, final QName processQName) {
    this.instance = instance;
    this.setBpelExecution(instance);
    this.processQName = processQName;
  }

  public Boolean execute(final Environment environment) throws Exception {
    ExecElementToSignal execElt = null;

    ProcessLock processLock = MutexRepository.lockProcess(processQName);
    try {
      InstanceLock lock = MutexRepository.lockInstance(instance);
      try {
        execElt = new FindExecutionToSignalCommand(instance).execute(environment);
      } finally {
        MutexRepository.unlockInstance(lock);
      }
    } finally {
      MutexRepository.unlockProcess(processLock);
    }
    if (execElt != null && !instance.isEnded()) {
      environment.get(MessageSession.class).send(new AsyncSignalExec(instance, processQName, execElt));
    }

    JobDbSession dbSession = environment.get(JobDbSession.class);
    if (dbSession==null) {
      throw new OrchestraRuntimeException("no "+JobDbSession.class.getName()+" in environment");
    }
    dbSession.delete(this);
    return true;
  }
}
