/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.services;

import javax.xml.namespace.QName;

import org.jbpm.pvm.env.Environment;
import org.jbpm.pvm.internal.jobexecutor.JobDbSession;
import org.jbpm.pvm.session.MessageSession;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.MutexRepository.InstanceLock;


public class AsyncSignalExec extends BpelMessageImpl<Boolean> {

  private ReceivingElement recvElt;

  protected AsyncSignalExec() { };

  public AsyncSignalExec(final BpelExecution instance, final QName processQName, final ExecElementToSignal execElementToSignal) {
    this.instance = instance;
    this.processQName = processQName;
    this.setBpelExecution(execElementToSignal.getBpelExecution());
    this.recvElt = execElementToSignal.getReceivingElement();
  }

  public Boolean execute(final Environment environment) throws Exception {
    ExecElementToSignal execElt = new ExecElementToSignal(getBpelExecution(), recvElt);
    InstanceLock lock = MutexRepository.lockInstance(instance);
    try {
      new SignalExecCommand(execElt).execute(environment);
    } finally {
      MutexRepository.unlockInstance(lock);
    }
    if (!instance.isEnded()) {
      Receiver.addWaitingExecutions(instance);
      environment.get(MessageSession.class).send(new AsyncFindExecToSignal(instance, processQName));
    }
    JobDbSession dbSession = environment.get(JobDbSession.class);
    if (dbSession==null) {
      throw new OrchestraRuntimeException("no "+JobDbSession.class.getName()+" in environment");
    }
    dbSession.delete(this);
    return true;
  }
}
