/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services;

import java.util.UUID;

import javax.transaction.Synchronization;

import org.jbpm.pvm.env.Environment;
import org.jbpm.pvm.env.Transaction;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.var.Message;

public class MessageCarrier {

  private final Object lock = new Object();
  
  private Message message;

  private boolean hasMessage = false;

  private String uuid;

  public MessageCarrier() {
    this.uuid = UUID.randomUUID().toString();
  }

  public Message getMessage() {
    synchronized (lock) {
      while (!hasMessage) {
        try {
          lock.wait();
        } catch (InterruptedException e) {
          throw new OrchestraRuntimeException(e);
        }
      }
      return message;
    }
  }

  public void setMessage(Message message) {
    synchronized (lock) {
      this.message = message.duplicate();
      hasMessage = true;
      final Transaction tx = Environment.getCurrent().get(Transaction.class);
      if (tx != null) {
        Synchronization synchro = new Synchronization() {
          public void beforeCompletion() { 
          }
          public void afterCompletion(int status) {
            if (status == javax.transaction.Status.STATUS_COMMITTED) {
              synchronized (MessageCarrier.this.lock) {
                MessageCarrier.this.lock.notifyAll();
              }
            }
          }
        };
        tx.registerSynchronization(synchro);
      } else { 
        this.lock.notifyAll();
      }
    }
  }
  
  public synchronized boolean hasMessage() {
    synchronized (lock) {
      return hasMessage;  
    }
  }

  public String getUuid() {
    return uuid;
  }

}
