/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services;

import org.ow2.orchestra.facade.data.runtime.ActivityInstanceData;
import org.ow2.orchestra.facade.data.runtime.CorrelationSetInitializedData;
import org.ow2.orchestra.facade.data.runtime.ForeachInitializedData;
import org.ow2.orchestra.facade.data.runtime.PartnerLinkUpdateData;
import org.ow2.orchestra.facade.data.runtime.ProcessInstanceData;
import org.ow2.orchestra.facade.data.runtime.TimerData;
import org.ow2.orchestra.facade.data.runtime.TransitionConditionUpdateData;
import org.ow2.orchestra.facade.data.runtime.VariableUpdateData;
import org.ow2.orchestra.facade.def.full.ProcessFullDefinition;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

/**
 * A Recorder is responsible for storing <strong>runtime</strong> instances
 * informations such as currently running instances and activities.
 *
 * Usually, recorded records are available using queries on a {@link Journal}.
 *
 * @author Alfonso Fuca
 *
 */
public interface Recorder {

  String DEFAULT_KEY = "recorder";

  /**
   * Record informations about process instance started
   *
   * @param processUUID the uuid of the process definition
   * @param instanceUUID the uuid of the process instance
   * @param processInstanceData data of the process instance
   */
  void recordInstanceStarted(ProcessInstanceData processInstanceData);

  /**
   * Record informations about process instance ended
   *
   * @param instanceUUID the uuid of the process instance
   * @param processInstanceData data of the process instance
   */
  void recordInstanceEnded(ProcessInstanceData processInstanceData);

  /**
   * Record informations about activity instance started
   *
   * @param processUUID the uuid of the process definition
   * @param instanceUUID the uuid of the process instance
   * @param activityUUID the uuid of the activity instance
   * @param encloserInstanceUUID the uuid of parent node execution it is null if
   *          we are the activity process
   * @param activityInstanceData data of the activity instance
   */
  void recordActivityStarted(ActivityInstanceData activityInstanceData);

  /**
   * Record informations about activity instance ended
   *
   * @param activityUUID the uuid of the activity instance
   * @param activityInstanceData data of the activity instance
   */
  void recordActivityEnded(ActivityInstanceData activityInstanceData);

  /**
   *
   * @param startValue
   * @param stopValue
   * @param completionConditionValue can be null
   */
  void recordForeachInitialized(ForeachInitializedData foreachInitializedData);

  /**
   * This method is called by wait and pick activities.
   *
   * @param timerData data
   */
  void recordTimerStarted(TimerData timerData);

  /**
   * Generic method for removing a journal record.
   *
   * @param archivable the record to archive
   */
  void remove(Archivable archivable);

  void recordVariableUpdated(VariableUpdateData variableUpdateData);

  void recordCorrelationSetInitialized(CorrelationSetInitializedData correlationSetInitializedData);

  void recordPartnerLinkUpdated(PartnerLinkUpdateData partnerLinkUpdateData);

  void recordTransitionConditionUpdated(TransitionConditionUpdateData transitionConditionUpdateData);

  /**
   * This metho record data about process definition deployed
   *
   * @param processActivityFullDefinitionImpl the deployed process to record
   */
  void recordProcessDeployed(ProcessFullDefinition processFullDefinition);

  /**
   * This metho record data about process definition undeployed
   *
   * @param processActivityFullDefinitionImpl the undeployed process to record
   */
  void recordProcessUndeployed(ProcessDefinitionUUID processDefinitionUUID);

}
