package org.ow2.orchestra.services.impl;

import java.util.List;

import javax.xml.namespace.QName;

import org.hibernate.FlushMode;
import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;
import org.jbpm.pvm.internal.hibernate.HibernatePvmDbSession;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.ScopeRuntime;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.services.Receiver;
import org.ow2.orchestra.services.ReplierKey;
import org.ow2.orchestra.services.db.hibernate.QNameUserType;
import org.ow2.orchestra.services.itf.BpelPersistenceService;
import org.ow2.orchestra.services.itf.Replier;

public class HibernateBpelPersistenceService extends
    HibernatePvmDbSession implements BpelPersistenceService {

  public HibernateBpelPersistenceService(final Session session) {
    setSession(session);
    session.setFlushMode(FlushMode.COMMIT);
  }

  public BpelProcess findBpelProcess(final String namespace, final String name, final int version) {
    Query query = getSession().getNamedQuery("findBpelProcess");
    query.setString("namespace", namespace);
    query.setString("name", name);
    query.setInteger("version", version);
    BpelProcess processDefinition = (BpelProcess) query.uniqueResult();
    return processDefinition;
  }

  public BpelProcess findBpelProcess(final String namespace, final String name) {
    Query query = getSession().getNamedQuery("findLatestBpelProcess");
    query.setString("namespace", namespace);
    query.setString("name", name);
    query.setMaxResults(1);
    BpelProcess processDefinition = (BpelProcess) query.uniqueResult();
    return processDefinition;
  }

  public List<BpelProcess> getBpelProcesses() {
    Query query = getSession().getNamedQuery("getBpelProcesses");
    return query.list();
  }

  public Receiver getReceiver(final OperationKey operationKey) {
    Query query = getSession().getNamedQuery("getReceiver");
    query.setParameter("processQName", operationKey.getProcessQName(),
        Hibernate.custom(QNameUserType.class));
    query.setParameter("portTypeQName", operationKey.getPortTypeQName(),
        Hibernate.custom(QNameUserType.class));
    query.setString("operationName", operationKey.getOperationName());
    return (Receiver) query.uniqueResult();
  }

  public List<Receiver> getReceivers(final QName processQName) {
    Query query = getSession().getNamedQuery("getReceivers");
    query.setParameter("processQName", processQName, Hibernate
        .custom(QNameUserType.class));
    return query.list();
  }

  public Replier getReplier(final ReplierKey replierKey) {
    Query query = getSession().getNamedQuery("getReplier");
    OperationKey operationKey = replierKey.getOperationKey();
    query.setParameter("processQName", operationKey.getProcessQName(),
        Hibernate.custom(QNameUserType.class));
    query.setParameter("portTypeQName", operationKey.getPortTypeQName(),
        Hibernate.custom(QNameUserType.class));
    query.setString("operationName", operationKey.getOperationName());
    query.setString("messageExchange", replierKey.getMessageExchange());
    query.setEntity("scope", replierKey.getScopeRuntime());
    return (Replier) query.uniqueResult();
  }

  public List<BpelExecution> getBpelInstances() {
    Query query = getSession().getNamedQuery("getBpelInstances");
    return query.list();
  }

  public BpelExecution getBpelInstance(final ProcessInstanceUUID processInstanceUUID) {
    Query query = getSession().getNamedQuery("getBpelInstance");
    query.setString("processInstanceUUID", processInstanceUUID.toString());
    query.setMaxResults(1);
    BpelExecution bpelExecution = (BpelExecution) query.uniqueResult();
    return bpelExecution;
  }

  public List<BpelExecution> getBpelInstances(final QName processQName) {
    Query query = getSession().getNamedQuery("getBpelInstancesOfProcess");
    query.setString("namespace", processQName.getNamespaceURI());
    query.setString("name", processQName.getLocalPart());
    return query.list();
  }

  public List<Replier> getRepliers(final ScopeRuntime scopeRuntime) {
    Query query = getSession().getNamedQuery("getRepliersByScopeRuntime");
    query.setEntity("scope", scopeRuntime);
    return query.list();
  }

}
